@@ -17,6 +17,12 @@ module Criterion
     -- * Creating a benchmark suite
     , Benchmark
     , env
+    , envWithCleanup
+    , perBatchEnv
+    , perBatchEnvWithCleanup
+    , perRunEnv
+    , perRunEnvWithCleanup
+    , toBenchmarkable
     , bench
     , bgroup
     -- ** Running a benchmark
