@@ -22,22 +22,24 @@ import qualified Data.Aeson as Aeson
 import Control.DeepSeq (rnf)
 import Control.Exception (evaluate)
 import Control.Monad (foldM, forM_, void, when, unless)
+import Control.Monad.Catch (MonadMask, finally)
 import Control.Monad.Reader (ask, asks)
 import Control.Monad.Trans (MonadIO, liftIO)
 import Control.Monad.Trans.Except
-import qualified Data.Binary as Binary 
+import qualified Data.Binary as Binary
 import Data.Int (Int64)
 import qualified Data.ByteString.Lazy.Char8 as L
 import Criterion.Analysis (analyseSample, noteOutliers)
 import Criterion.IO (header, headerRoot, critVersion, readJSONReports, writeJSONReports)
 import Criterion.IO.Printf (note, printError, prolix, writeCsv)
-import Criterion.Measurement (runBenchmark, secs)
+import Criterion.Measurement (runBenchmark, runBenchmarkable_, secs)
 import Criterion.Monad (Criterion)
 import Criterion.Report (report)
 import Criterion.Types hiding (measure)
+import Criterion.Types.Internal (fakeEnvironment)
 import qualified Data.Map as Map
 import qualified Data.Vector as V
-import Statistics.Resampling.Bootstrap (Estimate(..))
+import Statistics.Types (Estimate(..),ConfInt(..),confidenceInterval,cl95,confidenceLevel)
 import System.Directory (getTemporaryDirectory, removeFile)
 import System.IO (IOMode(..), hClose, openTempFile, openFile, hPutStr, openBinaryFile)
 import Text.Printf (printf)
@@ -79,10 +81,11 @@ analyseOne i desc meas = do
         _ <- bs r2 (regResponder ++ ":") regRSquare
         forM_ (Map.toList regCoeffs) $ \(prd,val) ->
           bs (printf "%.3g") ("  " ++ prd) val
-      writeCsv (desc,
-                estPoint anMean, estLowerBound anMean, estUpperBound anMean,
-                estPoint anStdDev, estLowerBound anStdDev,
-                estUpperBound anStdDev)
+      writeCsv
+        (desc,
+         estPoint anMean,   fst $ confidenceInterval anMean,   snd $ confidenceInterval anMean,
+         estPoint anStdDev, fst $ confidenceInterval anStdDev, snd $ confidenceInterval anStdDev
+        )
       when (verbosity == Verbose || (ovEffect > Slight && verbosity > Quiet)) $ do
         when (verbosity == Verbose) $ noteOutliers reportOutliers
         _ <- note "variance introduced by outliers: %d%% (%s)\n"
@@ -90,12 +93,16 @@ analyseOne i desc meas = do
         return ()
       _ <- note "\n"
       return (Analysed rpt)
-      where bs :: (Double -> String) -> String -> Estimate -> Criterion ()
-            bs f metric Estimate{..} =
+      where bs :: (Double -> String) -> String -> Estimate ConfInt Double -> Criterion ()
+            bs f metric e@Estimate{..} =
               note "%-20s %-10s (%s .. %s%s)\n" metric
-                   (f estPoint) (f estLowerBound) (f estUpperBound)
-                   (if estConfidenceLevel == 0.95 then ""
-                    else printf ", ci %.3f" estConfidenceLevel)
+                   (f estPoint) (f $ fst $ confidenceInterval e) (f $ snd $ confidenceInterval e)
+                   (let cl = confIntCL estError
+                        str | cl == cl95 = ""
+                            | otherwise  = printf ", ci %.3f" (confidenceLevel cl)
+                    in str
+                   )
+
 
 -- | Run a single benchmark and analyse its performance.
 runAndAnalyseOne :: Int -> String -> Benchmarkable -> Criterion DataRecord
@@ -122,7 +129,7 @@ runAndAnalyse select bs = do
   -- The type we write to the file is ReportFileContents, a triple.
   -- But here we ASSUME that the tuple will become a JSON array.
   -- This assumption lets us stream the reports to the file incrementally:
-  liftIO $ hPutStr handle $ "[ \"" ++ headerRoot ++ "\", " ++ 
+  liftIO $ hPutStr handle $ "[ \"" ++ headerRoot ++ "\", " ++
                              "\"" ++ critVersion ++ "\", [ "
 
   for select bs $ \idx desc bm -> do
@@ -139,7 +146,7 @@ runAndAnalyse select bs = do
     res <- readJSONReports jsonFile
     case res of
       Left err -> error $ "error reading file "++jsonFile++":\n  "++show err
-      Right (_,_,rs) -> 
+      Right (_,_,rs) ->
        case mbJsonFile of
          Just _ -> return rs
          _      -> removeFile jsonFile >> return rs
@@ -160,7 +167,7 @@ rawReport reports = do
     Just file -> liftIO $ do
       handle <- openBinaryFile file ReadWriteMode
       L.hPut handle header
-      forM_ reports $ \rpt ->  
+      forM_ reports $ \rpt ->
         L.hPut handle (Binary.encode rpt)
       hClose handle
 
@@ -175,20 +182,20 @@ runFixedIters :: Int64            -- ^ Number of loop iterations to run.
 runFixedIters iters select bs =
   for select bs $ \_idx desc bm -> do
     _ <- note "benchmarking %s\n" desc
-    liftIO $ runRepeatedly bm iters
+    liftIO $ runBenchmarkable_ bm iters
 
 -- | Iterate over benchmarks.
-for :: MonadIO m => (String -> Bool) -> Benchmark
+for :: (MonadMask m, MonadIO m) => (String -> Bool) -> Benchmark
     -> (Int -> String -> Benchmarkable -> m ()) -> m ()
 for select bs0 handle = go (0::Int) ("", bs0) >> return ()
   where
-    go !idx (pfx, Environment mkenv mkbench)
+    go !idx (pfx, Environment mkenv cleanenv mkbench)
       | shouldRun pfx mkbench = do
         e <- liftIO $ do
           ee <- mkenv
           evaluate (rnf ee)
           return ee
-        go idx (pfx, mkbench e)
+        go idx (pfx, mkbench e) `finally` liftIO (cleanenv e)
       | otherwise = return idx
     go idx (pfx, Benchmark desc b)
       | select desc' = do handle idx desc' b; return $! idx + 1
@@ -198,8 +205,7 @@ for select bs0 handle = go (0::Int) ("", bs0) >> return ()
       foldM go idx [(addPrefix pfx desc, b) | b <- bs]
 
     shouldRun pfx mkbench =
-      any (select . addPrefix pfx) . benchNames . mkbench $
-      error "Criterion.env could not determine the list of your benchmarks since they force the environment (see the documentation for details)"
+      any (select . addPrefix pfx) . benchNames . mkbench $ fakeEnvironment
 
 -- | Write summary JSON file (if applicable)
 json :: [Report] -> Criterion ()
