@@ -31,6 +31,12 @@ module Criterion.Main
     , Benchmark
     -- * Creating a benchmark suite
     , env
+    , envWithCleanup
+    , perBatchEnv
+    , perBatchEnvWithCleanup
+    , perRunEnv
+    , perRunEnvWithCleanup
+    , toBenchmarkable
     , bench
     , bgroup
     -- ** Running a benchmark
@@ -56,7 +62,8 @@ import Criterion.Main.Options (MatchType(..), Mode(..), defaultConfig, describe,
 import Criterion.Measurement (initializeTime)
 import Criterion.Monad (withConfig)
 import Criterion.Types
-import Data.List (isPrefixOf, sort, stripPrefix)
+import Data.Char (toLower)
+import Data.List (isInfixOf, isPrefixOf, sort, stripPrefix)
 import Data.Maybe (fromMaybe)
 import Options.Applicative (execParser)
 import System.Environment (getProgName)
@@ -97,6 +104,8 @@ makeMatcher matchKind args =
            Left errMsg -> Left . fromMaybe errMsg . stripPrefix "compile :: " $
                           errMsg
            Right ps -> Right $ \b -> null ps || any (`match` b) ps
+    Pattern -> Right $ \b -> null args || any (`isInfixOf` b) args
+    IPattern -> Right $ \b -> null args || any (`isInfixOf` map toLower b) (map (map toLower) args)
 
 selectBenches :: MatchType -> [String] -> Benchmark -> IO (String -> Bool)
 selectBenches matchType benches bsgroup = do
@@ -114,8 +123,8 @@ selectBenches matchType benches bsgroup = do
 -- > import Criterion.Main
 -- >
 -- > myConfig = defaultConfig {
--- >              -- Do not GC between runs.
--- >              forceGC = False
+-- >               -- Resample 10 times for bootstrapping
+-- >               resamples = 10
 -- >            }
 -- >
 -- > main = defaultMainWith myConfig [
@@ -145,9 +154,9 @@ runMode wat bs =
   case wat of
     List -> mapM_ putStrLn . sort . concatMap benchNames $ bs
     Version -> putStrLn versionInfo
-    RunIters iters matchType benches -> do
+    RunIters cfg iters matchType benches -> do
       shouldRun <- selectBenches matchType benches bsgroup
-      withConfig defaultConfig $
+      withConfig cfg $
         runFixedIters iters shouldRun bsgroup
     Run cfg matchType benches -> do
       shouldRun <- selectBenches matchType benches bsgroup
