@@ -57,8 +57,8 @@ module Criterion.Types
     , addPrefix
     , benchNames
     -- ** Evaluation control
-    , whnf
     , nf
+    , whnf
     , nfIO
     , whnfIO
     -- * Result types
@@ -72,12 +72,8 @@ module Criterion.Types
     , DataRecord(..)
     ) where
 
--- Temporary: to support pre-SMP
-import Data.Semigroup
-
 import Control.DeepSeq (NFData(rnf))
-import Control.Exception (evaluate)
-import Criterion.Types.Internal (fakeEnvironment)
+import Criterion.Types.Internal (fakeEnvironment, nf', whnf')
 import Data.Aeson (FromJSON(..), ToJSON(..))
 import Data.Binary (Binary(..), putWord8, getWord8)
 import Data.Data (Data, Typeable)
@@ -212,7 +208,7 @@ instance ToJSON Measured where
       (measTime, measCpuTime, measCycles, measIters,
        i measAllocated, i measNumGcs, i measBytesCopied,
        d measMutatorWallSeconds, d measMutatorCpuSeconds,
-       d measGcWallSeconds, d measMutatorCpuSeconds)
+       d measGcWallSeconds, d measGcCpuSeconds)
       where i = fromInt; d = fromDouble
 
 instance NFData Measured where
@@ -313,45 +309,60 @@ instance Binary Measured where
     get = Measured <$> get <*> get <*> get <*> get
                    <*> get <*> get <*> get <*> get <*> get <*> get <*> get
 
--- | Apply an argument to a function, and evaluate the result to weak
--- head normal form (WHNF).
-whnf :: (a -> b) -> a -> Benchmarkable
-whnf = pureFunc id
-{-# INLINE whnf #-}
-
 -- | Apply an argument to a function, and evaluate the result to
 -- normal form (NF).
 nf :: NFData b => (a -> b) -> a -> Benchmarkable
-nf = pureFunc rnf
-{-# INLINE nf #-}
+nf f x = toBenchmarkable (nf' rnf f x)
 
-pureFunc :: (b -> c) -> (a -> b) -> a -> Benchmarkable
-pureFunc reduce f0 x0 = toBenchmarkable (go f0 x0)
-  where go f x n
-          | n <= 0    = return ()
-          | otherwise = evaluate (reduce (f x)) >> go f x (n-1)
-{-# INLINE pureFunc #-}
+-- | Apply an argument to a function, and evaluate the result to weak
+-- head normal form (WHNF).
+whnf :: (a -> b) -> a -> Benchmarkable
+whnf f x = toBenchmarkable (whnf' f x)
 
 -- | Perform an action, then evaluate its result to normal form.
 -- This is particularly useful for forcing a lazy 'IO' action to be
 -- completely performed.
 nfIO :: NFData a => IO a -> Benchmarkable
-nfIO = toBenchmarkable . impure rnf
-{-# INLINE nfIO #-}
+nfIO a = toBenchmarkable (nfIO' rnf a)
 
 -- | Perform an action, then evaluate its result to weak head normal
 -- form (WHNF).  This is useful for forcing an 'IO' action whose result
 -- is an expression to be evaluated down to a more useful value.
 whnfIO :: IO a -> Benchmarkable
-whnfIO = toBenchmarkable . impure id
-{-# INLINE whnfIO #-}
+whnfIO a = toBenchmarkable (whnfIO' a)
 
-impure :: (a -> b) -> IO a -> Int64 -> IO ()
-impure strategy a = go
+-- Along with nf' and whnf', the following two functions are the core
+-- benchmarking loops. They have been carefully constructed to avoid
+-- allocation while also evaluating @a@.
+--
+-- These functions must not be inlined. There are two possible issues that
+-- can arise if they are inlined. First, the work is often floated out of
+-- the loop, which creates a nonsense benchmark. Second, the benchmark code
+-- itself could be changed by the user's optimization level. By marking them
+-- @NOINLINE@, the core benchmark code is always the same.
+--
+-- See #183 and #184 for discussion.
+
+-- | Generate a function that will run an action a given number of times,
+-- reducing it to normal form each time.
+nfIO' :: (a -> b) -> IO a -> (Int64 -> IO ())
+nfIO' reduce a = go
   where go n
           | n <= 0    = return ()
-          | otherwise = a >>= (evaluate . strategy) >> go (n-1)
-{-# INLINE impure #-}
+          | otherwise = do
+              x <- a
+              reduce x `seq` go (n-1)
+{-# NOINLINE nfIO' #-}
+
+-- | Generate a function that will run an action a given number of times.
+whnfIO' :: IO a -> (Int64 -> IO ())
+whnfIO' a = go
+  where
+    go n | n <= 0    = return ()
+         | otherwise = do
+             x <- a
+             x `seq` go (n-1)
+{-# NOINLINE whnfIO' #-}
 
 -- | Specification of a collection of benchmarks and environments. A
 -- benchmark may consist of:
@@ -523,7 +534,7 @@ perBatchEnvWithCleanup
     -- newly generated environment.
     -> Benchmarkable
 perBatchEnvWithCleanup alloc clean work
-    = Benchmarkable alloc clean (impure rnf . work) False
+    = Benchmarkable alloc clean (nfIO' rnf . work) False
 
 -- | Create a Benchmarkable where a fresh environment is allocated for every
 -- run of the operation to benchmark. This is useful for benchmarking mutable
@@ -709,9 +720,6 @@ instance NFData Regression where
 data SampleAnalysis = SampleAnalysis {
       anRegress    :: [Regression]
       -- ^ Estimates calculated via linear regression.
-    , anOverhead   :: Double
-      -- ^ Estimated measurement overhead, in seconds.  Estimation is
-      -- performed via linear regression.
     , anMean       :: St.Estimate St.ConfInt Double
       -- ^ Estimated mean.
     , anStdDev     :: St.Estimate St.ConfInt Double
@@ -726,12 +734,12 @@ instance ToJSON SampleAnalysis
 
 instance Binary SampleAnalysis where
     put SampleAnalysis{..} = do
-      put anRegress; put anOverhead; put anMean; put anStdDev; put anOutlierVar
-    get = SampleAnalysis <$> get <*> get <*> get <*> get <*> get
+      put anRegress; put anMean; put anStdDev; put anOutlierVar
+    get = SampleAnalysis <$> get <*> get <*> get <*> get
 
 instance NFData SampleAnalysis where
     rnf SampleAnalysis{..} =
-        rnf anRegress `seq` rnf anOverhead `seq` rnf anMean `seq`
+        rnf anRegress `seq` rnf anMean `seq`
         rnf anStdDev `seq` rnf anOutlierVar
 
 -- | Data for a KDE chart of performance.
@@ -760,9 +768,7 @@ data Report = Report {
     , reportKeys     :: [String]
       -- ^ See 'measureKeys'.
     , reportMeasured :: V.Vector Measured
-      -- ^ Raw measurements. These are /not/ corrected for the
-      -- estimated measurement overhead that can be found via the
-      -- 'anOverhead' field of 'reportAnalysis'.
+      -- ^ Raw measurements.
     , reportAnalysis :: SampleAnalysis
       -- ^ Report analysis.
     , reportOutliers :: Outliers
